    /** Plot the run-length encoded text file:
     *                   picture.txt
     *  The format of the file should be as follows
     *     image-width image-height
     *     count color count color count color ... (Image Line 1)
     *     count color count color count color ... (Image Line 2)
     *     ...
     *     count color count color count color ... (Image Line N)
     *     -1 (Negative count Terminator)
     *
     *  Read the color table to be used with the picture from
     *  the picture file.
     */
class PictureMain4 {
    public static void main(String[] args){
        // An input reader.
        SimpleInput pictureFile = new SimpleInput("picture2.txt");
        // Read the picture width and height from the file.
        int width = pictureFile.nextInt();
        int height = pictureFile.nextInt();
        // Create the plotter.
        PicturePlotter plotter = new PicturePlotter(width,height);

        // Read the number of colors in the color table.
        int numColors = pictureFile.nextInt();
        // There are three RGB values per color.
        final int NUM_RGB = 3;
        // Create the color table to be used with the picture.
        int[][] table = new int[numColors][NUM_RGB];

        // Read the component values.
        for(int color = 0; color < numColors; color++){
            for(int component = 0; component < NUM_RGB; component++){
                table[color][component] = pictureFile.nextInt();
            }
        }
        // Set the color table to be used by the plotter.
        plotter.setColorTable(table);

        // Keep track of the next (x,y) position to be plotted.
        int x = 0, y = 0;
        // Read the count for the next color.
        int count = pictureFile.nextInt();
        while(count > 0){
            // Read the color to go with the count.
            int color = pictureFile.nextInt();
            plotter.plot(x,y,count,color);
            // Update the x position on this line.
            x += count;
            // See if we have reached the end of the line?
            if(x >= width){
                // Set up for the next line.
                x = 0;
                y++;
            }
            // Read the count for the next color.
            count = pictureFile.nextInt();
        }
        // Allow the plotter to refresh its image.
        plotter.repaint();
    }
}
