        // Act as an intelligent player. The game
        // manager requests row and col values
        // via getRow and getCol, after first calling chooseSquare.
class PlayToWin {
    public PlayToWin(TTTGame game,char player){
        this.game = game;
        this.player = player;
    }

        // Choose a (row,col) combination for the next move.
        // Assume that at least one square is free.
        // Set the row and col attributes for subsequent
        // retrieval via their accessors.
    public void chooseSquare(){
        final TTTGame game = getGame();
        final int size = game.getBoardSize();
        // Obtain a copy of the board.
        final char[][] board = game.getBoardCopy();

        // Set out-of-bounds values for (row,col) to indicate that
        // no choice has yet been made.
        setRow(-1);
        setCol(-1);

        // Calculate unique sums for each row, column and
        // the two diagonals, based upon the current occupants
        // of each grid square.
        int leftDiagonal = 0, rightDiagonal = 0;

        // The value needed for a winning line.
        final int winningLine = (size-1)*myValue;
        // The value of a line needing blocking, in order to
        // prevent an opponent from winning.
        final int shouldBlock = (size-1)*opponentValue;

        for(int x = 0; x < size; x++){
            int rowSum = 0, colSum = 0;
            for(int y = 0; y < size; y++){
                rowSum += squareValue(board[x][y]);
                colSum += squareValue(board[y][x]);
            }
            // See if we have found a winning row.
            if((rowSum == winningLine) || (rowSum == shouldBlock)){
                // There should be a single free square to complete
                // the row.
                for(col = 0; !selectIfFree(board,x,col); col++){
                }
                if(rowSum == winningLine){
                    return;
                }
            }
            // See if we have found a winning col.
            if((colSum == winningLine) || (colSum == shouldBlock)){
                // There should be a single free square to complete
                // the column.
                for(row = 0; !selectIfFree(board,row,x); row++){
                }
                if(colSum == winningLine){
                    return;
                }
            }
            leftDiagonal += squareValue(board[x][x]);
            rightDiagonal += squareValue(board[x][size-x-1]);
        }
        // See if we have found a winning diagonal.
        if((leftDiagonal == winningLine) ||
                                (leftDiagonal == shouldBlock)){
            for(int row = 0; !selectIfFree(board,row,row); row++){
            }
            if(leftDiagonal == winningLine){
                return;
            }
        }
        if((rightDiagonal == winningLine) ||
                                (rightDiagonal == shouldBlock)){
            for(int row = 0; !selectIfFree(board,row,size-row-1); row++){
            }
            if(rightDiagonal == winningLine){
                return;
            }
        }
        // No immediately winning lines available.
        // See if a blocking move has been set.
        if(getCol() >= 0){
            // One has.
        }
        else{
            // Nothing set so far.
            // Try the center.
            int row = size/2;
            if(selectIfFree(board,row,row)){
                // It is free.
            }
            // Look for a free corner.
            else if(selectIfFree(board,0,0)){
                // It is free.
            }
            else if(selectIfFree(board,0,size-1)){
            }
            else if(selectIfFree(board,size-1,0)){
            }
            else if(selectIfFree(board,size-1,size-1)){
            }
            else{
                // Still nothing found. Return the first free square.
                // NB. This strategy is not optimal, because
                // we could explicitly seek to create size-1 in a line.
                for(int x = 0; x < size; x++){
                    for(int y = 0; y < size; y++){
                        if(selectIfFree(board,x,y)){
                            return;
                        }
                        else if(selectIfFree(board,y,x)){
                            return;
                        }
                    }
                }
                // This should be impossible.
                System.out.println("Failed to find a free square.");
            }
        }
    }

        // Return the row as result, and save the col for
        // return by the next call to getCol().
    public int getRow(){
        return row;
    }

        // Return the col set by the immediately preceding call
        // to getRow().
    public int getCol(){
        return col;
    }

    public char getPlayer(){
        return player;
    }

    public void setPlayer(char p){
        player = p;
    }

        // Return a value for the given occupant of a square,
        // based upon whether the piece is this player's, the 
        // opponents, or if the square is free.
    private int squareValue(char occupant){
        if(occupant == getPlayer()){
            return myValue;
        }
        else if(occupant == TTTGame.Free){
            return 0;
        }
        else{
            return opponentValue;
        }
    }

        // If (row,col) is free, return true and set the
        // row and col attributes. Otherwise, return false.
    private boolean selectIfFree(char[][] board,int row,int col){
        if(board[row][col] == TTTGame.Free){
            setRow(row);
            setCol(col);
            return true;
        }
        else{
            return false;
        }
    }

    private void setRow(int r){
        row = r;
    }

    private void setCol(int c){
        col = c;
    }

    private TTTGame getGame(){
        return game;
    }

    // Square values for this player and the opponent.
    // These are prime numbers. These particular values have
    // been chosen to try to avoid multiples of one looking
    // like multiples of the other.
    private static final int myValue = 13, opponentValue = 17;
    // A row and column value; both are determined by getRow().
    private int row, col;
    // The current game state.
    private final TTTGame game;
    // Which symbol is being played, Nought or Cross.
    private char player;
}
