        // Run a game of noughts and crosses.
        // One side is taken by a PlayToWin object.
class TTTStrategyManager {
        // How big the game's board should be.
    public TTTStrategyManager(int size){
        game = new TTTGame(size);
        strategy = new PlayToWin(game,strategyPiece);
        // Create a display for the game.
        display = new TTTDisplay(size);
    }

        // Play a single game, interacting with an opponent who
        // is playing to win.
    public void play(){

        final TTTGame game = getGame();
        game.clearTheBoard();
        getDisplay().clear();

        // Who plays next.
        char player = TTTGame.Nought;
        while(game.gameState() == TTTGame.Unfinished){
            if(player == strategyPiece){
                makeAStrategyMove(player);
            }
            else{
                makeARandomMove(player);
            }
            if(!game.hasWon(player)){
                player = game.nextPlayer(player);
            }
        }
        reportTheResult();
    }

        // Make a random move for player.
    private void makeARandomMove(char player){
        final TTTGame game = getGame();
        // We shall generate a random row and column for each move.
        int row, col;
        // Generate a valid random move.
        do{
            row = getRandomPosition();
            col = getRandomPosition();
        } while(!game.isSquareFree(row,col));
        // Play in the chosen square.
        makeAMove(row,col,player);
    }

        // Make a random move for player.
    private void makeAStrategyMove(char player){
        final PlayToWin strategy = getStrategy();
        final TTTGame game = getGame();
        // We shall generate a random row and column for each move.
        int row, col;
        // Generate a valid random move.
        do{
            // Request the player to choose their square.
            strategy.chooseSquare();
            // Retrieve the row and column.
            row = strategy.getRow();
            col = strategy.getCol();
        } while(!game.isSquareFree(row,col));
        // Play in the chosen square.
        makeAMove(row,col,player);
    }

        // Implement the given move for player.
        // (row,col) is assumed to be free.
    private void makeAMove(int row, int col, char player){
        getGame().play(row,col,player);
        getDisplay().showSquare(row,col,player);
    }

    private void reportTheResult(){
        final TTTGame game = getGame();
        int result = game.gameState();

        if(result == TTTGame.Drawn){
            System.out.println("The game was drawn.");
        }
        else if(result == TTTGame.NoughtWon){
            System.out.println(TTTGame.Nought+" won.");
        }
        else if(result == TTTGame.CrossWon){
            System.out.println(TTTGame.Cross+" won.");
        }
        else{
            System.out.println("Unknown result: "+result);
        }
    }

    private PlayToWin getStrategy(){
        return strategy;
    }

    // Return a valid random number as part of a move.
    private int getRandomPosition(){
        return (int) (Math.random() * getGame().getBoardSize());
    }

    private TTTGame getGame(){
        return game;
    }

    private TTTDisplay getDisplay(){
        return display;
    }

    // The board. Initialized in the constructor.
    private final TTTGame game;
    // A GUI for the game.
    private final TTTDisplay display;
    // The piece the strategy player will use.
    private final char strategyPiece = TTTGame.Cross;
    private final PlayToWin strategy;
}
