import java.util.*;

        // Provide dealing functionality for a multi-player
        // version of snap. The number of players participating
        // is passed to dealTheCards.
class Dealer {
    public static final int minimumNumPlayers = 2;

        // Create a new pack and distribute cards to the players.
    public void dealTheCards(int numPlayers){
        // Create the appropriate number of players for the game.
        setupPlayers(numPlayers);
        final Player[] players = getPlayers();

        // Distribute the cards fairly so that each
        // players has the same number of cards.
        // Discard any remainder.
        final Pack pack = new Pack();
        pack.shuffle();
        while(pack.size() >= numPlayers){
            for(int nextPlayer = 0; nextPlayer < numPlayers; nextPlayer++){
                Card card = pack.nextCard();
                players[nextPlayer].takeCard(card);
            }
        }
    }

        // The cards have been distributed; play a game.
    public void superviseAGame(){
        final Player[] players = getPlayers();
        final int numPlayers = players.length;
        // Where to show the game.
        CardDisplay display = new CardDisplay(numPlayers);
        display.clear();

        // Keep a copy of who is still in the game.
        // When a player's cards are exhausted, they are removed
        // from this collection.
        final LinkedList stillInGame = new LinkedList();
        for(int i = 0; i < numPlayers; i++){
            stillInGame.add(players[i]);
        }

        // The player in stillInGame who is due to play 
        // the next card.
        int nextPlayer = 0;

        // Create a pile for the first round.
        Pile pile = new Pile();

        // Play until there are fewer than 2 players remaining.
        while(stillInGame.size() >= 2){
            // Select the player.
            Player player = (Player) stillInGame.get(nextPlayer);
            // Request a card.
            Card cardPlayed = player.playCard();
            if(cardPlayed != null){
                display.showCard(cardPlayed,player.getIdentity());
                // Add it to the pile.
                pile.addCard(cardPlayed);
                if(pile.snap()){
                    display.report("Snap!"+
                                   "Player: "+player.getIdentity()+" wins "+
                                   pile.numCards()+" card(s).");
                    display.clear();
                    // player wins the cards in the pile.
                    player.takePile(pile);
                    // Create a new pile for the next round.
                    pile = new Pile();
                }
                else{
                    // Select the next player for this round.
                    nextPlayer = (nextPlayer+1) % stillInGame.size();
                }
            }
            else{
                // Remove this player from the game.
                display.report("Player: "+player.getIdentity()+
                               " is out of the game.");
                display.clear();
                stillInGame.remove(nextPlayer);
                if(nextPlayer >= stillInGame.size()){
                    nextPlayer = 0;
                }
            }
        }
        Player winner = (Player) stillInGame.get(0);
        display.report("Player: "+winner.getIdentity()+" has won.");
    }

    private void setupPlayers(int numPlayers){
        // Set up the game's players.
        // Force a minimum number of players.
        if(numPlayers < minimumNumPlayers){
            numPlayers = minimumNumPlayers;
        }
        this.players = new Player[numPlayers];
        for(int i = 0; i < numPlayers; i++){
            // Pass the player their identifying number.
            this.players[i] = new Player(i);
        }
    }

    private Player[] getPlayers(){
        return players;
    }

    // The players are set up as a result of calling
    // deal the cards.
    private Player[] players = new Player[0];
}
