import java.util.*;

        // Store an ordered list of items.
        // Each item stored must implement the Priority interface.
        // Items with higher priorities are stored with lower
        // indices than those with lower priorities.
class PriorityList {
        // Add a Priority implementing item to the list.
        // Items with equal priority are ordered with earlier
        // arrivals having lower indices.
    public void add(Object item) throws RuntimeException {
        if(item instanceof Priority){
            // Ok to handle it.
            int priority = ((Priority) item).getPriority();
            List list = getList();
            final int numItems = list.size();
            if(numItems == 0){
                list.add(item);
            }
            else{
                // Test for insertion at the head of the list.
                Priority head = (Priority) list.get(0);
                // Iterate over the list to find an appropriate
                // location.
                boolean locationFound = false;
                ListIterator iterator = list.listIterator();
                while(!locationFound && iterator.hasNext()){
                    Priority next = (Priority) iterator.next();
                    if(priority > next.getPriority()){
                        locationFound = true;
                    }
                }
                if(locationFound){
                    // Go back one place.
                    iterator.previous();
                    iterator.add(item);
                }
                else{
                    // Add it at the end.
                    list.add(item);
                }
            }
        }
        else{
            throw new RuntimeException(
                "Only classes implementing Priority are allowed.");
        }
    }

        // Return (and remove) the item at the head of the list.
    public Priority next(){
        List list = getList();
        if(list.size() > 0){
            return (Priority) list.remove(0);
        }
        else{
            return null;
        }
    }

    public void listPriorities(){
        List list = getList();
        Iterator it = list.iterator();
        System.out.println("The list of priorities is as follows:");
        while(it.hasNext()){
            Priority p = (Priority) it.next();
            System.out.print(p.getPriority()+" ");
        }
        System.out.println();
    }

        // Assume that the items have a toString method.
    public void showList(){
        List list = getList();
        Iterator it = list.iterator();
        System.out.println("The list of items is as follows:");
        while(it.hasNext()){
            Priority p = (Priority) it.next();
            System.out.print(p.toString()+" ");
        }
        System.out.println();
    }

    private List getList(){
        return list;
    }

    // Use the fact that both LinkedList and ArrayList implement
    // the List interface to hide details of the actual type used
    // from the rest of the class.
    private final List list = new LinkedList();
}
