import java.io.*;

        /* Provide for restoring and saving of a Serializable
         * League. Without an argument, a new League is created
         * and stored in League.obj.
         * With a single filename argument, an attempt is made
         * to restore a saved League from the file. If the file
         * does not exist, it is used to save a newly created
         * League. Each time the program is run, points
         * are added to each team.
         */
class LeagueMain {
    public static void main(String[] args){
        try{
            // The default name of the file in which to save the league.
            String filename = "League.obj";
            League league = null;

            if(args.length > 1){
                System.err.println("Usage: java LeagueMain [file]");
                System.exit(1);
            }
            else if(args.length == 1){
                // If the file exists, try to restore a saved version.
                File details = new File(args[0]);
                if(details.exists()){
                    league = readSavedLeague(args[0]);
                    if(league != null){
                        filename = args[0];
                    }
                    else{
                        // There were problems.
                        // Use the default file name.
                    }
                }
                else{
                    // Use the given file name for saving.
                    filename = args[0];
                }
            }
            // else - create from scratch.

            if(league == null){
                league = new League(new TeamDetails[]{
                    new TeamDetails("Kestrels", "Able Amy", "Ambleside"),
                    new TeamDetails("Eagles", "Brave Ben", "Big Rock"),
                    new TeamDetails("Hawks", "Capable Charis", "Cheam"),
                    new TeamDetails("Falcons", "Dull Dan", "Ditchfield"),
                });
            }

            // Print the state of the league before changing it.
            System.out.println("======== Before ========");
            league.print();

            // Give points to the teams.
            league.addPoints("Kestrels",3);
            league.addPoints("Eagles",1);
            league.addPoints("Hawks",3);
            league.addPoints("Falcons",5);

            // Print the state of the league before serialization.
            System.out.println("======== After ========");
            league.print();
            league.save(filename);
        }
        catch(IOException e){
            System.err.println("IO Exception: "+e.getMessage());
        }
    }

        // Try to read a saved version of a League object
        // from filename. Propagate any IOExceptions.
    public static League readSavedLeague(String filename)
                                    throws IOException {
        League league = null;
        ObjectInputStream istream = null;
        try{
            istream = new ObjectInputStream(
                            new FileInputStream(filename));
            league = (League) istream.readObject();
        }
        catch(ClassNotFoundException e){
            System.err.println(e.getMessage());
        }
        finally{
            if(istream != null){
                istream.close();
            }
        }
        return league;
    }
}
