        /* Try to guess a 5-digit number selected by the user.
         * Request the user to indicate whether each guess is too high,
         * too low or correct.
         * The method used is to keep track of the lower and upper
         * limits of the range of values that is still valid before
         * each guess. As a result of the user's response, the range
         * is reduced. The computer always picks a guess in the middle
         * of the valid range.
         */
class ComputerPlayer {
    public static void main(String[] args){
        SimpleInput keyboard = new SimpleInput();
        // The minimum and maximum allowed guesses.
        final int min = 10000, max = 99999;
        // Values for the user's responses to the guess.
        final int CORRECT = 0, TOOLOW = 1, TOOHIGH = 2;
        int response;

        // Keep track of the lowest and highest values it could be,
        // based upon the user's responses. These values are
        // moved towards each other after each invalid guess.
        int lowestPossible = min, highestPossible = max;

        // Keep track of how many guesses where needed.
        int numGuesses = 0;
        System.out.println("Think of a 5-digit number in the range "+
                           min+" to "+max);
        do{
            // Choose a guess in the middle of the possible range.
            int guess = (lowestPossible+highestPossible)/2;
            System.out.println("My guess is: "+guess);
            numGuesses++;
            // Insist that the user response with 0, 1 or 2.
            do{
                // Read the user's response to the guess.
                System.out.println("Is it correct ("+CORRECT+"), "+
                                   "too low ("+TOOLOW+") or "+
                                   "too high ("+TOOHIGH+")?");
                response = keyboard.nextInt();
            } while((response < CORRECT) || (response > TOOHIGH));

            // Adjust the valid range according to the response.
            if(response == TOOLOW){
                // Try the higher end of the range.
                lowestPossible = guess+1;
            }
            else if(response == TOOHIGH){
                // Try the lower end of the range.
                highestPossible = guess-1;
            }
            // else - the guess must be correct.
        } while((response != CORRECT) &&
            (lowestPossible <= highestPossible));

        if(response == CORRECT){
            System.out.println("I took "+numGuesses+" guess(es)");
        }
        else{
            System.out.println("Something was wrong with your responses.");
        }
    }
}
