    /** Plot the run-length encoded text file:
     *                   picture.txt
     *  The format of the file should be as follows
     *     image-width image-height
     *     count color count color count color ... (Image Line 1)
     *     count color count color count color ... (Image Line 2)
     *     ...
     *     count color count color count color ... (Image Line N)
     *     -1 (Negative count Terminator)
     *
     *  This version attempts to optimize the number of calls to
     *  plot, by combining consecutive occurrences of the same
     *  color on a single line.
     *  The approach it uses is to delay plotting a color until
     *  the next (count,color) pair has been read.
     */
class PictureMain2 {
    public static void main(String[] args){
        // An input reader.
        SimpleInput pictureFile = new SimpleInput("picture.txt");
        // Read the picture width and height from the file.
        int width = pictureFile.nextInt();
        int height = pictureFile.nextInt();
        // Create the plotter.
        PicturePlotter plotter = new PicturePlotter(width,height);

        // Keep track of the next (x,y) position to be plotted.
        int x = 0, y = 0;
        // Read the first (count,color) pair, but delay plotting them.
        int lastCount = pictureFile.nextInt();
        int lastColor = pictureFile.nextInt();

        // Read the second count for the second color.
        int count = pictureFile.nextInt();
        while(count > 0){
            // Read the color to go with the count.
            int color = pictureFile.nextInt();
            // See if the end of the line has been reached.
            if((x+lastCount) >= width){
                // End of line. Plot, even if colors match.
                plotter.plot(x,y,lastCount,lastColor);
                x = 0;
                y++;
                // Delay plotting the current pair.
                lastCount = count;
                lastColor = color;
            }
            else if(color == lastColor){
                // Combine this pair with the last pair.
                lastCount += count;
                // Don't plot yet.
            }
            else{
                // Color mismatch. Plot what we have.
                plotter.plot(x,y,lastCount,lastColor);
                x += lastCount;
                if(x >= width){
                    x = 0;
                    y++;
                }
                // Delay plotting the current pair.
                lastCount = count;
                lastColor = color;
            }
            // Read the count for the next color.
            count = pictureFile.nextInt();
        }
        // Make sure any (lastCount,lastColor) pair has been written.
        if(lastCount > 0){
            plotter.plot(x,y,lastCount,lastColor);
        }
        // Allow the plotter to refresh its image.
        plotter.repaint();
    }
}
